@@ -44,6 +44,21 @@
 #include "main.h"
 #include "read-files.h"
 
+#ifdef USE_LDAP
+#ifdef HAVE_LBER_H
+#include <lber.h>
+#endif
+#ifdef HAVE_LDAP_H
+#include <ldap.h>
+#endif
+#ifdef HAVE_LDAP_SSL_H
+#include <ldap_ssl.h>
+#endif
+
+#define _INCLUDED_FROM_LDAPCFN_C_
+#include "ldapfcn.h"
+#endif
+
 #ifndef _
 #define _(String) gettext (String)
 #endif
@@ -67,7 +82,7 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 static void
 print_usage (FILE *stream, const char *program)
 {
-  fprintf (stream, _("Usage: %s [-D binddn] [-m mindays][-M maxdays][-d lastday][-I inactive][-E expiredate][-W warndays] user\n"),
+  fprintf (stream, _("Usage: %s [-D binddn][-P path][-m mindays][-M maxdays][-d lastday][-I inactive][-E expiredate][-W warndays] user\n"),
 	   program);
   fprintf (stream, _("       %s -l user\n"),
 	   program);
@@ -79,8 +94,13 @@ print_help (const char *program)
   print_usage (stdout, program);
   fprintf (stdout, _("%s - change user password expiry information\n\n"),
 	   program);
+
+#ifdef USE_LDAP
   fputs (_("  -D binddn      Use dn \"binddn\" to bind to the LDAP directory\n"),
          stdout);
+#endif
+  fputs (_("  -P path        Search passwd and shadow file in \"path\"\n"),
+	 stdout);
 
   fputs (_("  -r service     Use nameservice 'service'\n"), stdout);
   fputs (_("  -q, --quiet    Don't be verbose\n"), stdout);
@@ -331,7 +351,9 @@ main (int argc, char *argv[])
   char *caller_name = NULL;
   char *mindays = NULL, *maxdays = NULL, *lastday = NULL, *inactive = NULL;
   char *expiredate = NULL, *warndays = NULL;
+#ifdef USE_LDAP
   char *binddn = NULL;
+#endif
   int interactive = 1;
   int silent = 0;
   int l_flag = 0;
@@ -358,7 +380,9 @@ main (int argc, char *argv[])
 	{"expiredate", required_argument, NULL, 'E' },
 	{"warndays",   required_argument, NULL, 'W' },
 	{"list",       no_argument,       NULL, 'l' },
+#ifdef USE_LDAP
 	{"binddn",     required_argument, NULL, 'D' },
+#endif
 	{"quiet",      no_argument,       NULL, 'q' },
 	{"path",       required_argument, NULL, 'P' },
 	{"service",    required_argument, NULL, 'r' },
@@ -398,9 +422,11 @@ main (int argc, char *argv[])
 	  warndays = optarg;
 	  interactive = 0;
 	  break;
+#ifdef USE_LDAP
 	case 'D':
 	  binddn = optarg;
 	  break;
+#endif
 	case 'l':
 	  l_flag = 1;
 	  break;
@@ -429,8 +455,10 @@ main (int argc, char *argv[])
 	    use_service = "nisplus";
 	  else if (strcasecmp (optarg, "files") == 0)
 	    use_service = "files";
+#ifdef USE_LDAP
 	  else if (strcasecmp (optarg, "ldap") == 0)
 	    use_service = "ldap";
+#endif
 	  else
 	    {
 	      fprintf (stderr, _("Service \"%s\" not supported\n"), optarg);
@@ -520,8 +548,11 @@ main (int argc, char *argv[])
     }
 
   /* Only root is allowed to change aging for local users. */
-  if (!l_flag && uid && (pw_data->service == S_LOCAL ||
-			 (pw_data->service == S_LDAP && binddn == NULL)))
+  if (!l_flag && uid && (pw_data->service == S_LOCAL
+#ifdef USE_LDAP
+			 || (pw_data->service == S_LDAP && binddn == NULL)
+#endif
+			 ))
     {
       fprintf (stderr,
 	       _("Only an administrator is allowed to change aging information.\n"));
@@ -529,18 +560,107 @@ main (int argc, char *argv[])
       return E_NOPERM;
     }
 
+#ifdef USE_LDAP
   if (binddn && pw_data->service == S_LDAP && !l_flag)
     {
       /* A user tries to change data stored in a LDAP database and
 	 knows the Manager dn, now we need the password from him.  */
       char prompt[130+strlen (binddn)], *cp;
+      ldap_session_t *session = create_ldap_session (LDAP_PATH_CONF);
+      int rc, msgid;
+      struct timeval timeout;
+      LDAPMessage *result;
+#if defined(HAVE_LDAP_PARSE_RESULT) && defined(HAVE_LDAP_CONTROLS_FREE)
+      int parserc;
+#endif
+#ifdef HAVE_LDAP_CONTROLS_FREE
+      LDAPControl **controls;
+#endif
+
+      if (session == NULL)
+	return E_FAILURE;
 
-      pw_data->binddn = strdup (binddn);
       snprintf (prompt, sizeof (prompt), _("Enter LDAP Password:"));
       cp = getpass (prompt);
+
+      pw_data->binddn = strdup (binddn);
+
+      if (open_ldap_session (session) != 0)
+	return E_FAILURE;
+
+      msgid = ldap_simple_bind (session->ld, binddn, cp);
+      if (msgid == -1)
+	{
+#if 0 /* XXX */
+	  fprintf (stderr, "ldap_simple_bind %s",
+		   ldap_err2string (ldap_get_lderrno (session->ld, 0, 0)));
+#endif
+	  _pam_overwrite (cp);
+	  return E_NOPERM;
+	}
+
+      timeout.tv_sec = 10;
+      timeout.tv_usec = 0;
+      rc = ldap_result (session->ld, msgid, FALSE, &timeout, &result);
+      if (rc == -1 || rc == 0)
+	{
+	  fprintf (stderr, "ldap_result %s\n",
+		   ldap_err2string (ldap_get_lderrno (session->ld, 0, 0)));
+	  _pam_overwrite (cp);
+	  return E_NOPERM;
+	}
+
+#if defined(HAVE_LDAP_PARSE_RESULT) && defined(HAVE_LDAP_CONTROLS_FREE)
+      controls = 0;
+      parserc =
+	ldap_parse_result (session->ld, result, &rc, 0, 0, 0, &controls, TRUE);
+      if (parserc != LDAP_SUCCESS)
+	{
+	  if (isatty (fileno (stderr)))
+	    fprintf (stderr, "ldap_parse_result %s\n",
+		     ldap_err2string (parserc));
+	  else
+	    syslog (LOG_ERR, "ldap_parse_result %s",
+		    ldap_err2string (parserc));
+	  _pam_overwrite (session->info->userpw);
+	  _pam_drop (session->info->userpw);
+	  return parserc;
+	}
+
+      if (controls != NULL)
+	{
+	  LDAPControl **ctlp;
+	  for (ctlp = controls; *ctlp != NULL; ctlp++)
+	    {
+	      if (!strcmp ((*ctlp)->ldctl_oid, LDAP_CONTROL_PWEXPIRING))
+		{
+		  char seconds[32];
+		  snprintf (seconds, sizeof seconds, "%.*s",
+			    (int) (*ctlp)->ldctl_value.bv_len,
+			    (*ctlp)->ldctl_value.bv_val);
+		  session->info->password_expiration_time = atol (seconds);
+		}
+	      else if (!strcmp ((*ctlp)->ldctl_oid, LDAP_CONTROL_PWEXPIRED))
+		{
+		  session->info->password_expired = 1;
+		  _pam_overwrite (session->info->userpw);
+		  _pam_drop (session->info->userpw);
+		  rc = LDAP_SUCCESS;
+		  /* That may be a lie, but we need to get to the acct_mgmt
+		   * step and force the change...
+		   */
+		}
+	    }
+	  ldap_controls_free (controls);
+	}
+#else
+      rc = ldap_result2error (session->ld, result, TRUE);
+#endif
+
       pw_data->oldclearpwd = strdup (cp);
     }
   else
+#endif /* USE_LDAP */
     /* misuse l_flag: We don't need to extra ask for a password with "-l".  */
     if (do_authentication (program, caller_name, pw_data, !l_flag) != 0)
       return E_NOPERM;
@@ -557,7 +677,7 @@ main (int argc, char *argv[])
 	{
 	  syslog (LOG_ERR, "%s: failed to drop privileges: %s\n",
 		  program, strerror (errno));
-	  fprintf (stderr, "%s: failed to drop privileges: %s\n",
+	  fprintf (stderr, _("%s: failed to drop privileges: %s\n"),
 		   program, strerror (errno));
 	  return E_FAILURE;
         }
@@ -573,7 +693,11 @@ main (int argc, char *argv[])
 
   /* Caller must be root or he needs to know the binddn and password
      for LDAP administrator.  */
-  if (uid != 0 && !(binddn && pw_data->service == S_LDAP))
+  if (uid != 0
+#ifdef USE_LDAP
+      && !(binddn && pw_data->service == S_LDAP)
+#endif
+      )
     return E_USAGE;
 
   if (interactive)
@@ -681,8 +805,10 @@ main (int argc, char *argv[])
   else
     pw_data->sp_changed = TRUE;
 
+#ifdef USE_LDAP
   if (binddn)
     pw_data->binddn = strdup (binddn);
+#endif
 
   if (do_setpwnam (pw_data) != 0)
     {
