@@ -19,6 +19,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifndef _LDAPFCN_H_
+#define _LDAPFCN_H_
+
+#ifdef USE_LDAP
+
 /* /etc/ldap.conf nss_ldap-style configuration */
 typedef struct ldap_config
   {
@@ -202,11 +207,9 @@ ldap_config_t *read_ldap_config (const char *);
 void free_ldap_config (ldap_config_t **pconfig);
 ldap_session_t *create_ldap_session (const char *configFile);
 int open_ldap_session (ldap_session_t *);
-#if 0
 #ifndef HAVE_LDAP_GET_LDERRNO
 int ldap_get_lderrno (LDAP * ld, char **m, char **s);
 #endif
-#endif
 int get_ldapuser_info (ldap_session_t *, const char *);
 int ldap_authentication (ldap_session_t *, const char *, const char *);
 int ldap_update_shell (ldap_session_t *session, const char *user,
@@ -243,3 +246,6 @@ if (X) { \
 #ifndef TRUE
 #define TRUE !FALSE
 #endif
+
+#endif /* USE_LDAP */
+#endif /* _LDAPFCN_H_ */
