@@ -324,7 +324,7 @@ handle_responses (SSL *ssl)
 /* Load the config file (/etc/rpasswd.conf)  */
 static int
 load_config (const char *configfile, int verbose, int check_syntax,
-	     char **hostp, char **portp)
+	     char **hostp, char **portp, int *reqcertp)
 {
   FILE *fp;
   char *buf = NULL;
@@ -405,13 +405,31 @@ load_config (const char *configfile, int verbose, int check_syntax,
 	    }
 	  continue;
 	}
+      else if (strncmp (cp, "reqcert", 7) == 0 && isspace ((int)cp[7]))
+	{
+	  char *p = &cp[7];
+
+	  while (isspace (*p))
+	    ++p;
+
+	  if (strcmp (p, "never") == 0)
+	    *reqcertp = 0;
+	  else if (strcmp (p, "allow") == 0)
+	    *reqcertp = 1;
+	  else if (strcmp (p, "try") == 0)
+	    *reqcertp = 2;
+	  else if (strcmp (p, "demand") == 0 ||
+		   strcmp (p, "hard") == 0)
+	    *reqcertp = 3;
+	  continue;
+	}
       if (check_syntax)
         {
           printf (_("Entry \"%s\" is not valid!\n"), cp);
           ++bad_entries;
         }
       else
-        fprintf (stderr, _("Entry \"%s\" is not valid, ignore it!"), cp);
+        fprintf (stderr, _("Entry \"%s\" is not valid, ignored!\n"), cp);
     }
   fclose (fp);
 
@@ -451,6 +469,7 @@ main (int argc, char **argv)
   int sock = -1, ipv4 = 1, ipv6 = 1;
   int verbose = 0;
   int admin_mode = 0;
+  int reqcert = 3;
   int retval;
   char *username;
   SSL_CTX *ctx;
@@ -548,14 +567,14 @@ main (int argc, char **argv)
 
   if (hostp)
     if (portp)
-      load_config (config_file, verbose, 0, NULL, NULL);
+      load_config (config_file, verbose, 0, NULL, NULL, &reqcert);
     else
-      load_config (config_file, verbose, 0, NULL, &portp);
+      load_config (config_file, verbose, 0, NULL, &portp, &reqcert);
   else
     if (portp)
-      load_config (config_file, verbose, 0, &hostp, NULL);
+      load_config (config_file, verbose, 0, &hostp, NULL, &reqcert);
     else
-      load_config (config_file, verbose, 0, &hostp, &portp);
+      load_config (config_file, verbose, 0, &hostp, &portp, &reqcert);
 
   if (portp == NULL)
     portp = "rpasswd";
@@ -683,10 +702,11 @@ Hostname or service not known for specified protocol\n"));
     X509 *server_cert;
     char *str;
     SSL_METHOD *meth;
+    long verify_result;
     int err;
 
     SSLeay_add_ssl_algorithms ();
-    meth = SSLv2_client_method ();
+    meth = SSLv23_client_method ();
     SSL_load_error_strings ();
     ctx = SSL_CTX_new (meth);
     if (ctx == NULL)
@@ -695,6 +715,25 @@ Hostname or service not known for specified protocol\n"));
 	return E_SSL_FAILURE;
       }
 
+#if 0
+    /* This is only necessary if we configure a unusual path.
+       XXX Make this a program option.  */
+    if (!SSL_CTX_load_verify_locations (ctx, NULL, "/etc/ssl/certs"))
+      {
+	fprintf (stderr, _("error loading default verify locations: %s\n"),
+		 ERR_error_string (ERR_get_error (), NULL));
+	if (reqcert > 1)
+	  return E_SSL_FAILURE;
+      }
+#endif
+    if (!SSL_CTX_set_default_verify_paths(ctx))
+      {
+	fprintf (stderr, _("error setting default verify path: %s\n"),
+		 ERR_error_string (ERR_get_error (), NULL));
+	if (reqcert > 1)
+	  return E_SSL_FAILURE;
+      }
+
     /* Now we have TCP conncetion. Start SSL negotiation. */
     ssl = SSL_new (ctx);
     if (ssl == NULL)
@@ -711,41 +750,62 @@ Hostname or service not known for specified protocol\n"));
 	return E_SSL_FAILURE;
       }
 
-    /* Following two steps are optional and not required for
-       data exchange to be successful.  */
-    if (verbose)
+    if (reqcert > 0 || verbose)
       {
-	/* Get the cipher.  */
-	printf (_("SSL connection using %s\n\n"), SSL_get_cipher (ssl));
-
 	/* Get server's certificate (note: beware of dynamic allocation).  */
 	server_cert = SSL_get_peer_certificate (ssl);
-	if (server_cert == NULL)
-	  fprintf (stderr, _("Server does not have a certificate?\n"));
-	else
+
+	/* Verify severs certificate.  */
+	verify_result = SSL_get_verify_result (ssl);
+
+	/* Following two steps are optional and not required for
+	   data exchange to be successful except the client couldn't verfiy
+	   the server certificate.  */
+	if (verify_result || verbose)
 	  {
-	    printf (_("Server certificate:\n"));
+	    /* Get the cipher.  */
+	    printf (_("SSL connection using %s\n\n"), SSL_get_cipher (ssl));
 
-	    str =
-	      X509_NAME_oneline (X509_get_subject_name (server_cert), 0, 0);
-	    if (str)
+	    if (server_cert == NULL)
 	      {
-		printf (_("  subject: %s\n"), str);
-		free (str);
+		fprintf (stderr, _("Server does not have a certificate?\n"));
+		if (reqcert >= 3)
+		  return E_SSL_FAILURE;
 	      }
-	    str =
-	      X509_NAME_oneline (X509_get_issuer_name (server_cert), 0, 0);
-	    if (str)
+	    else
 	      {
-		printf (_("  issuer: %s\n"), str);
-		free (str);
-	      }
-	    /* We could do all sorts of certificate verification stuff
-	       here before deallocating the certificate.  */
+		printf (_("Server certificate:\n"));
 
-	    X509_free (server_cert);
-	    fputs ("\n", stdout);
+		str = X509_NAME_oneline (X509_get_subject_name (server_cert),
+					 0, 0);
+		if (str)
+		  {
+		    printf (_("  subject: %s\n"), str);
+		    free (str);
+		  }
+		str = X509_NAME_oneline (X509_get_issuer_name (server_cert),
+					 0, 0);
+		if (str)
+		  {
+		    printf (_("  issuer: %s\n"), str);
+		    free (str);
+		  }
+		/* We could do all sorts of certificate verification stuff
+		   here before deallocating the certificate.  */
+
+		fputs ("\n", stdout);
+	      }
 	  }
+
+	if ((verify_result = SSL_get_verify_result (ssl)) != X509_V_OK)
+	  {
+	    fprintf (stderr, "Server certificate is not ok: %s!\n",
+		     X509_verify_cert_error_string (verify_result));
+	    if (reqcert >= 2)
+	      return E_SSL_FAILURE;
+	  }
+
+	X509_free (server_cert);
       }
   }
 
