@@ -1082,6 +1082,37 @@ main (int argc, char **argv)
   if (check_pid (_PATH_RPASSWDDPID))
     error (EXIT_FAILURE, 0, _("already running"));
 
+  /* Initialize SSL data. We need to do this before we go in
+     background, else we cannot read the PEM phass phrase.  */
+  SSL_load_error_strings ();
+  SSLeay_add_ssl_algorithms ();
+  meth = SSLv23_server_method ();
+  ctx = SSL_CTX_new (meth);
+  if (!ctx)
+    {
+      dbg_log (ERR_error_string (ERR_get_error (), NULL));
+      return E_SSL_FAILURE;
+    }
+
+  if (SSL_CTX_use_certificate_file (ctx, certificate, SSL_FILETYPE_PEM) <= 0)
+    {
+      dbg_log ("Loading certificate (%s): %s", certificate,
+	       ERR_error_string (ERR_get_error (), NULL));
+      return E_SSL_FAILURE;
+    }
+  if (SSL_CTX_use_PrivateKey_file (ctx, privatekey, SSL_FILETYPE_PEM) <= 0)
+    {
+      dbg_log ("Loading privatekey (%s): %s", privatekey,
+	       ERR_error_string (ERR_get_error (), NULL));
+      return E_SSL_FAILURE;
+    }
+
+  if (!SSL_CTX_check_private_key (ctx))
+    {
+      dbg_log (ERR_error_string (ERR_get_error (), NULL));
+      return E_SSL_FAILURE;
+    }
+
   /* Behave like a daemon.  */
   if (go_background)
     {
@@ -1126,36 +1157,6 @@ main (int argc, char **argv)
   /* Set the limits to a usefull value.  */
   init_limits ();
 
-  /* Initialize SSL data.  */
-  SSL_load_error_strings ();
-  SSLeay_add_ssl_algorithms ();
-  meth = SSLv23_server_method ();
-  ctx = SSL_CTX_new (meth);
-  if (!ctx)
-    {
-      dbg_log (ERR_error_string (ERR_get_error (), NULL));
-      return E_SSL_FAILURE;
-    }
-
-  if (SSL_CTX_use_certificate_file (ctx, certificate, SSL_FILETYPE_PEM) <= 0)
-    {
-      dbg_log ("Loading certificate (%s): %s", certificate,
-	       ERR_error_string (ERR_get_error (), NULL));
-      return E_SSL_FAILURE;
-    }
-  if (SSL_CTX_use_PrivateKey_file (ctx, privatekey, SSL_FILETYPE_PEM) <= 0)
-    {
-      dbg_log ("Loading privatekey (%s): %s", privatekey,
-	       ERR_error_string (ERR_get_error (), NULL));
-      return E_SSL_FAILURE;
-    }
-
-  if (!SSL_CTX_check_private_key (ctx))
-    {
-      dbg_log (ERR_error_string (ERR_get_error (), NULL));
-      return E_SSL_FAILURE;
-    }
-
   /* If port was not specified on commandline, try at first a lookup
      in the service database, if this fails, use the compiled in
      default port.  */
