@@ -126,6 +126,8 @@ free_user_t (user_t *data)
     free (data->new_gecos);
   if (data->oldclearpwd)
     free (data->oldclearpwd);
+  if (data->binddn)
+    free (data->binddn);
   free (data);
 }
 
@@ -718,19 +720,52 @@ Error while closing temporary shadow file: %m\n"));
       if (session != NULL)
 	{
 	  if (data->new_shell)
-	    retval = ldap_update_shell (session, data->pw.pw_name,
-					data->oldclearpwd, data->new_shell);
+	    retval = ldap_update_field (session, data->pw.pw_name,
+					data->binddn, data->oldclearpwd,
+					"loginShell", data->new_shell);
 	  else if (data->new_gecos)
-	    retval = ldap_update_gecos (session, data->pw.pw_name,
-					data->oldclearpwd, data->new_gecos);
+	    retval = ldap_update_field (session, data->pw.pw_name,
+					data->binddn, data->oldclearpwd,
+					"gecos", data->new_gecos);
+	  else if (data->sp_changed)
+	    {
+	      char buf[200];
+
+	      snprintf (buf, sizeof (buf), "%ld", data->spn.sp_min);
+	      retval = ldap_update_field (session, data->pw.pw_name,
+					  data->binddn, data->oldclearpwd,
+					  "shadowMin", buf);
+	      snprintf (buf, sizeof (buf),"%ld", data->spn.sp_max);
+	      retval = ldap_update_field (session, data->pw.pw_name,
+					   data->binddn, data->oldclearpwd,
+					   "shadowMax", buf);
+	      snprintf (buf, sizeof (buf),"%ld", data->spn.sp_warn);
+	      retval = ldap_update_field (session, data->pw.pw_name,
+					   data->binddn, data->oldclearpwd,
+					   "shadowWarning", buf);
+	      snprintf (buf, sizeof (buf),"%ld", data->spn.sp_inact);
+	      retval = ldap_update_field (session, data->pw.pw_name,
+					   data->binddn, data->oldclearpwd,
+					   "shadowInactive", buf);
+	      snprintf (buf, sizeof (buf),"%ld", data->spn.sp_lstchg);
+	      retval = ldap_update_field (session, data->pw.pw_name,
+					   data->binddn, data->oldclearpwd,
+					   "shadowLastChange", buf);
+	      snprintf (buf, sizeof (buf),"%ld", data->spn.sp_expire);
+	      retval = ldap_update_field (session, data->pw.pw_name,
+					   data->binddn, data->oldclearpwd,
+					   "shadowExpire", buf);
+	    }
 	  else
 	    return -1;
 
 	  if (retval != 0)
 	    fprintf (stderr,
-		     _("LDAP login shell information update failed: %s\n"),
+		     _("LDAP information update failed: %s\n"),
 		     ldap_err2string (retval));
 	}
+      else
+	return -1;
     }
 #endif
   else
